using Benner.NaiveService.API.Infrastructure.Entities;
using Benner.NaiveService.API.Models;
using Benner.NaiveService.API.Services;
using Microsoft.AspNetCore.Mvc;

namespace Benner.NaiveService.API.Controllers
{
    [ApiController]
    [Route("[controller]")]
    public class ExemplosController : ControllerBase
    {
        private readonly IExemplosService _estadosService;

        public ExemplosController(IExemplosService estadosService)
        {
            _estadosService = estadosService;
        }

        [HttpPost]
        [Route("Criar")]
        public ActionResult Criar(string nomeEstado)
        {
            try
            {
                _estadosService.Criar(nomeEstado);
                return Ok();
            }
            catch (Exception e)
            {
                return BadRequest(e);
            }
        }

        [HttpPatch]
        [Route("AlterarNome")]
        public ActionResult AlterarNome(long handle, string nomeEstado)
        {
            try
            {
                _estadosService.AlterarNome(handle, nomeEstado);
                return Ok();
            }
            catch (Exception e)
            {
                return BadRequest(e);
            }
        }

        [HttpGet]
        [Route("RetornarTodos")]
        public ActionResult<IEnumerable<IEstados>> RetornarTodos()
        {
            try
            {
                return Ok(_estadosService.RetornarTodos());
            }
            catch (Exception e)
            {
                return BadRequest(e);
            }
        }

        [HttpDelete]
        [Route("Excluir")]
        public ActionResult Excluir(long handle)
        {
            try
            {
                _estadosService.Excluir(handle);
                return Ok();
            }
            catch (Exception e)
            {
                return BadRequest(e);
            }
        }

        [HttpGet]
        [Route("ObterQuantidadeCidadesHomonimas")]
        public ActionResult<MunicipioModel> ObterQuantidadeCidadesHomonimas(string nomeCidade)
        {
            try
            {
                return Ok(_estadosService.RetornarQuantidadeCidadesHomonimas(nomeCidade));
            }
            catch (Exception e)
            {
                return BadRequest(e);
            }
        }

        [HttpGet]
        [Route("RetornarCidadeHomonimaMaiorReincidenciaPorEstado")]
        public ActionResult<MunicipioModel> RetornarCidadeHomonimaMaiorReincidenciaPorEstado(long handleEstado)
        {
            try
            {
                var municipio = _estadosService.RetornarCidadeHomonimaMaiorReincidenciaPorEstado(handleEstado);
                return Ok(municipio);
            }
            catch (Exception e)
            {
                return BadRequest(e);
            }
        }
    }
}