﻿using Benner.Core.Bef;
using Benner.NaiveService.API.Infrastructure.Entities;
using Benner.NaiveService.API.Models;

namespace Benner.NaiveService.API.Services
{
    public class ExemplosService : IExemplosService
    {
        private readonly IDbContext _dbContext;

        public ExemplosService(IDbContext dbContext)
        {
            _dbContext = dbContext ?? throw new ArgumentNullException(nameof(dbContext));
        }

        public IEnumerable<IEstados> RetornarTodos()
        {
            using (var context = _dbContext.Clone())
            {
                var estados = context
                    .Query<IEstados>()
                    .ToList();

                return estados;
            }
        }

        public void Criar(string name)
        {
            using (var context = _dbContext.Clone())
            {
                var estado = context.CreateNew<IEstados>();
                estado.Nome = name;
                context.Save(estado);
            }
        }

        public IEstados AlterarNome(long handle, string name)
        {
            using (var context = _dbContext.Clone())
            {
                var estado = context.Query<IEstados>().Where(x => x.Handle == handle).ForEdit().First();
                estado.Nome = name;

                context.Save(estado);
                return estado;
            }
        }

        public void Excluir(long handle)
        {
            using (var context = _dbContext.Clone())
            {
                context.DeleteByHandle<IEstados>(handle);
            }
        }

        public MunicipioModel RetornarQuantidadeCidadesHomonimas(string cityName)
        {
            using (var context = _dbContext.Clone())
            {
                var município = context.Query<IMunicipios>()
                       .Where(x => x.Nome == cityName)
                       .Select<MunicipioModel>(mapping =>
                       {
                           mapping.Field(x => x.Handle, m => m.Quantidade, FieldFunctionOptions.Count);
                           mapping.Field(x => x.Nome, m => m.Nome, FieldFunctionOptions.GroupBy);
                       }).First();

                return município;
            }
        }

        public MunicipioModel RetornarCidadeHomonimaMaiorReincidenciaPorEstado(long handleEstado)
        {
            using (var context = _dbContext.Clone())
            {
                var lista = context
                    .Query<IEstados>()
                    .Join<IMunicipios>((estado, municipio) => municipio.Estado == estado.Handle)
                    .Where(estado => estado.Handle == handleEstado)
                    .Select<MunicipioModel>(mapping =>
                    {
                        mapping
                        .Field<IMunicipios>(municipio => municipio.Handle, m => m.Quantidade, FieldFunctionOptions.Count)
                        .Field<IMunicipios>(municipio => municipio.Nome, m => m.Nome, FieldFunctionOptions.GroupBy);
                    })
                    .ToList();

                return (from query in lista
                        where query.Quantidade == lista.Select(x => x.Quantidade).Max()
                        select new MunicipioModel
                        {
                            Nome = query.Nome,
                            Quantidade = query.Quantidade
                        }).First();
            }
        }
    }
}
